
package com.star.payroll;

import javax.ejb.*;
import javax.jms.*;
import javax.naming.InitialContext;

import com.wombat.benefits.DeductionUpdateBean;

import com.star.payroll.Payroll;
import com.star.payroll.PayrollHome;

/**
 * An implementation of the DeductionUpdateBean command bean interface
 * which was defined by Wombat. This implementation uses the PayrollQueue
 * JMS Queue to send payroll deduction information 
 * to Star Enterprise's payroll system in a loosely coupled manner.
 */
public class DeductionUpdateBeanJMS implements DeductionUpdateBean {

    private Queue payrollQueue;
    private QueueConnectionFactory factory;
    private int emplNumber;
    private double deduction;

    public DeductionUpdateBeanJMS() {
	try {
	    InitialContext ictx = new InitialContext();
	    payrollQueue = (Queue)ictx.lookup("java:comp/env/jms/PayrollQueue");
	    factory = (QueueConnectionFactory)ictx.lookup(
				    "java:comp/env/jms/QueueConnectionFactory");
	} catch ( Exception ex ) {
	    throw new EJBException("Unable to get Payroll bean", ex);
	}
    }

    public void setBenefitsDeduction(int emplNumber, double deduction) {
	this.emplNumber = emplNumber; 
	this.deduction = deduction;
    }

    public void execute() {
	QueueConnection connection=null;
	try {
	    // Create a connection
	    connection = factory.createQueueConnection();

	    // Create a transactional session
            QueueSession session = connection.createQueueSession(true, 0);

	    // Create a sender
            QueueSender sender = session.createSender(payrollQueue);

            // Send the message.
            MapMessage message = session.createMapMessage();
	    message.setInt("Employee", emplNumber);
	    message.setDouble("BenefitsDeduction", deduction);

            sender.send(message);
 
	} catch (Exception ex) {
	    throw new EJBException(ex);
        } finally {
            try {
                if( connection != null ) {
                    connection.close();
                }
            } catch(Exception e) {}
        }
    }

    public void release() {
	payrollQueue = null;
	factory = null;	
    }
}

